import tangram.CfgApp
import tangram.DATE
import tangram.Log
import tangram.Num
import tangram.SQLcmd
import tangram.SQLconn
import tangram.SQLfilter
import tangram.TangramJsonWriter
import tangram.http.TangramHttpRequest

import java.time.LocalDate

// zoznam tovarov s aktualnou cenou (GET), query-parametre:
// [store=XXX] kod predajne (povinny)
// [minutes=nn] len zmenene zaznamy za poslednych n minut (ak sa neuvedie, bezi full-export)
urlPriceList (sql, request, json)

private static void urlPriceList (SQLconn sql, TangramHttpRequest request, TangramJsonWriter json)
{
    // Parametre z URL a konfiguracie (Tangram.ini -> static cache)
    String store = request.queryMap.getString('store').toUpperCase()
    int minutes = request.queryMap.getInt('minutes')
    String censkup = CfgApp.eshopPriceGroup

    String tovfilter = 'WHERE T.Neaktivna is null and (A.xx is not null or B.Xx is not null)'
    if (minutes) tovfilter = tovfilter + """
        and (exists (select null FROM TOVARY X WHERE X.Xx=T.Xx and nvl(X.Xud,X.Xid) > (sysdate - interval '$minutes' minute))
          or exists (select null FROM TOVCENY X WHERE X.Kod=T.Xx and nvl(X.Xud,X.Xid) > (sysdate - interval '$minutes' minute)))
    """
    // Log.debug ("store=$store, minutes=$minutes, filter=$tovfilter\ncena='$cenfilter'")
    // Nacitanie riadkov z tabulky TOVARY (29.09.2022 pridany priznak F_100g)
    GString select = """
    SELECT T.Xx, T.Nazov, T.mj, T.Jmj, T.Jkoef, T.Povod, T.Pozn, T.Znacka, T.Zlozenie, T.Prodskup, T.Druh, T.Obal, T.F_100g, T.F_cena30hide,
        PRICE.Cm_sdph "price", PRICE.Datum_od "DatBegin", PRICE.Datum_do "DatEnd", PRICE.Akcia "Akcia",
        BASIC.Cm_sdph "priceBasic", BASIC.Akcia "akciaBasic",
        PACK.Cm_sdph "pricePack",
        S.Nazov "nameOfOrigin"
    FROM TOVARY T 
    LEFT JOIN
    ( SELECT Kod, max(Xx) KEEP (DENSE_RANK FIRST ORDER BY Cm_sdph) as Xx FROM TOVCENY
      WHERE Censkup='$censkup' and (trunc(sysdate) between Datum_od and Datum_do) and (Typceny='B' or (Typceny='D' and Adresa='$store'))
      GROUP BY Kod
    ) A ON (A.Kod=T.Xx)
    LEFT JOIN
    ( SELECT Kod, max(Xx) KEEP (DENSE_RANK FIRST ORDER BY Cm_sdph) as Xx FROM TOVCENY
      WHERE Censkup='$censkup' and (trunc(sysdate) between Datum_od and Datum_do) and Typceny='A'
      GROUP BY Kod
    ) B ON (B.Kod=T.Xx)
    LEFT JOIN
    ( SELECT Kod, min(Cm_sdph) as Cm_sdph FROM TOVCENY
      WHERE Censkup='$censkup' and (trunc(sysdate) between Datum_od and Datum_do) and Typceny='A'
      GROUP BY Kod
    ) PACK ON (PACK.Kod=T.Obal)

    LEFT JOIN TOVCENY PRICE ON (PRICE.Xx=A.Xx)
    LEFT JOIN TOVCENY BASIC ON (BASIC.Xx=B.Xx)
    LEFT JOIN C_STATY S ON (S.Xx=T.Povod)
    $tovfilter
    ORDER BY T.Xx
    """

	Log.debug (select)

    SQLcmd rows = sql.queryRows(select)

    // Response header
    request.sendHeaderJSON()

    // Reponse body
    int count = 0
    json.begin()
    json.arrayBegin('items')
    while (rows.next())
    {
        // dopocitane hodnoty
        String kod = rows.getString('xx')
        String mj  = rows.getString('mj')
        BigDecimal basic = rows.getBigDecimal('priceBasic')
        BigDecimal price = rows.getBigDecimal('price')
        String akcia = rows.getString('akcia')
        BigDecimal discount = null
        if (Num.isNullOrZero(price))
        {
            price = basic
            akcia = rows.getString('akciaBasic')
        }
        else 
		{
			if (! Num.isNullOrZero(basic))
                discount = ((basic - price) / basic).movePointRight(2).round(1)
		}

        // jednotkova cena
        BigDecimal jkoef = rows.getBigDecimalOrNull('jkoef')
        BigDecimal unitPrice = Num.isNullOrZero(jkoef) ? null : (price / jkoef).round(2)

        // 25.06.2024 minimalna cena za poslednych 30 dni
        LocalDate dateBegin = rows.getDate('datbegin')
        boolean cena30show = rows.getString('f_cena30hide').isEmpty()
        BigDecimal cena30 = (cena30show && dateBegin!=null) ? calcCena30 (sql, kod, censkup, dateBegin, store) : null

        // prepocet cien na 100g (ak je zadany priznak pre 100g ceny)
        if (rows.getString('f_100g'))
        {
            mj = "100g"
            if (price != null) price = price.movePointLeft(1).round(2)
            if (basic != null) basic = basic.movePointLeft(1).round(2)
            if (cena30!= null) cena30 = cena30.movePointLeft(1).round(2)
        }

        Log.debug("kod=$kod, basic=$basic, price=$price, min30=$cena30")

        // zapis do JSON
        json.blockBegin()

        json.pair('sku', kod)
        json.pair('sale_unit', mj)
        json.pair('store', store)
        json.pair('name', rows.getString('nazov'))
        json.pair('unit_price', unitPrice)
        json.pair('unit', rows.getString('jmj'))
        json.pair('place_of_origin', rows.getString('povod'))
        json.pair('name_of_origin', rows.getString('nameOfOrigin'))
        json.pair('price', price)
        json.pair('price_old', basic)
        json.pair('discount_value', discount)
        json.pair('description', rows.getString('pozn'))
        json.pair('brand', rows.getString('znacka'))
        json.pair('spec', rows.getString('zlozenie'))
        json.pair('typ_template', akcia)
        json.pair('date_begin', dateBegin)
        json.pair('date_end', rows.getDate('datend'))
        json.pair('icon', rows.getString('prodskup'))
        json.pair('category', rows.getString('druh'))
        json.pair('grade', 'Trieda II.')
        json.pair("sku_pack", rows.getString('obal'))
        json.pair("rp_price", rows.getBigDecimalOrNull('pricePack'))
        json.pair('price30', cena30)

        // pole ciarovych kodov
        exportBarcodes(sql, json, kod)

        // koniec bloku (jeden tovar)
        json.blockEnd()
        json.flush()
        count++
    }
    json.arrayEnd()
    json.end()

    rows.close()
    Log.info("Prices exported $count")
}

// export ciarovych kodov
private static void exportBarcodes (SQLconn sql, TangramJsonWriter json, String kod)
{
    // zoznam ciarovych kodov do zoznamu
    ArrayList barcodes = []
    SQLcmd rows = sql.queryRows("SELECT Barcode FROM TOVBARCODE WHERE Kod='$kod' ORDER BY Barcode")
    while (rows.next()) barcodes.add(rows.getString('barcode'))
    rows.close()

    // export zoznamu do JSON
    json.array(barcodes, "ean")
}

// vyhladanie najnizsej ceny za 30 dni pred zaciatkom platnosti akciovej ceny
/*
!!CB!! 70
! Hladame vsetky ceny v rovnakej CS, ktore zacali skor nez tato cena a platili v poslednych 30 dnoch
Call FLT.StrExact ("kod", Kod)
Call FLT.StrExact ("censkup", Censkup)
Call FLT.Add ("datum_od < " || SqlDate(Datum_od))
Call FLT.Add ("datum_do >= " || SqlDate(Datum_od - 30))
Call FLT.IsNull ("f_cena30ignore")
Call FLT.Add ("(typceny in ('A','B') or (typceny='C' and adrtyp=" || Q(ADRESA.Typ) || ") or (typceny='D' and adresa=" || Q(ADRESA.Xx) || "))")
Call SqlImmediate ("SELECT min(cm_sdph) FROM TOVCENY INTO :nCena30 WHERE " || FLT.Result())
 */
private static BigDecimal calcCena30 (SQLconn sql, String kod, String censkup, LocalDate dateBegin, String store)
{
    // ak sa nenasla akciova cena, tak nehladame minimalnu cenu
    if (dateBegin == null) return null

    SQLfilter flt = new SQLfilter()
    flt.strExact("kod", kod)
    flt.strExact("censkup", censkup)
    flt.add("datum_od < " + DATE.toSQL(dateBegin))
    flt.add("datum_do >= " + DATE.toSQL(dateBegin.minusDays(30)))
    flt.add("f_cena30ignore is null")
    flt.add("(typceny in ('A','B') or (typceny='D' and adresa='$store'))")
    Log.debug("MIN CENA " + flt.value())
    return sql.queryValueBigDecimal('SELECT min(cm_sdph) FROM TOVCENY WHERE ' + flt.value())
}

